<?php

namespace Config;

// Create a new instance of our RouteCollection class.
$routes = Services::routes();

/*
 * --------------------------------------------------------------------
 * Router Setup
 * --------------------------------------------------------------------
 */
$routes->setDefaultNamespace('App\Controllers');
$routes->setDefaultController('Home');
$routes->setDefaultMethod('index');
$routes->setAutoRoute('true');
$routes->setTranslateURIDashes(false);
$routes->set404Override();
$routes->useSupportedLocalesOnly(true);
// The Auto Routing (Legacy) is very dangerous. It is easy to create vulnerable apps
// where controller filters or CSRF protection are bypassed.
// If you don't want to define all routes, please use the Auto Routing (Improved).
// Set `$autoRoutesImproved` to true in `app/Config/Feature.php` and set the following to true.
// $routes->setAutoRoute(false);

/*
 * --------------------------------------------------------------------
 * Route Definitions
 * --------------------------------------------------------------------
 */

// We get a performance increase by specifying the default
// route since we don't have to scan directories.
$routes->get('/', 'Home::index');

$routes->get('/v2', 'Home::version');
$routes->get('/v2/(:num)', 'Home::version/$1');
$routes->get('/v2/(:num)/(:any)', 'Home::version/$1/$2');

$routes->get('login', 'Login::index');
$routes->post('login', 'Login::do_login');
$routes->get('logout', 'Login::logout');
$routes->get('success', 'Register::success');


// Logged In user routes .
$routes->group('',['filter' => 'registercheck'], function($routes){
        $routes->get('dashboard','Dashboard::index');
        
        $routes->get('pages','Pages::view');
        $routes->get('page/(:num)', 'Pages::view/$1');
        $routes->get('page-add','Pages::add');
        $routes->post('page-store','Pages::store');
        $routes->get('page/edit/(:num)','Pages::edit/$1');
        $routes->post('page/update/(:num)','Pages::update/$1');
        $routes->get('page/delete/(:num)','Pages::delete/$1');
        $routes->post('page-search', 'Pages::search');

        $routes->get('products','Product::view');
        $routes->get('product/(:num)', 'Product::view/$1');
        $routes->get('product-add','Product::add');
        $routes->post('product-store','Product::store');
        $routes->get('product/edit/(:num)','Product::edit/$1');
        $routes->post('product/update/(:num)','Product::update/$1');
        $routes->get('product/delete/(:num)','Product::delete/$1');
        
        $routes->get('employees','Employee::index');
        $routes->get('employee', 'Employee::index');
        $routes->get('employee/(:num)', 'Employee::index/$1');
        $routes->get('employee-add','Employee::create');
        $routes->post('employee-store','Employee::store');
        $routes->get('employee/edit/(:num)','Employee::edit/$1');
        $routes->post('employee-update/(:num)','Employee::update/$1');
        $routes->delete('employee/delete/(:num)','Employee::delete/$1');
        
        $routes->get('students','Student::index');
        $routes->get('student', 'Student::index');
        $routes->get('student/(:num)', 'Student::index/$1');
        $routes->get('student-add','Student::add_student_view');
        $routes->post('student-store','Student::add_student');
        $routes->get('student/edit/(:num)','Student::update_student_view/$1');
        $routes->post('student-update/(:num)','Student::update_student/$1');
        $routes->delete('student/delete/(:num)','Student::delete_student/$1');
        
        $routes->get('list','Register::do_list');
        $routes->get('register', 'Register::index');
        $routes->post('register', 'Register::do_register');
        $routes->get('edit','Register::do_edit');
        $routes->post('edit','Register::do_update');
        $routes->post('imageuser', 'Register::uploadImage');

});

      $routes->post('/galeri','Gallery::index');
      $routes->get('galeri/upload', 'Gallery::save',['as' => 'upload-galeri']);
      $routes->get('galeri/hapus/(:num)', 'Gallery::delete/$1',['as' => 'delete-galeri']);

        $routes->get('/supplier','Supplier::index');
        $routes->get('create-supplier','Supplier::create');
        $routes->post('create-supplier','Supplier::save');
        $routes->get('edit-suuplier/(:any)', 'Supplier::edit/$1');
        $routes->get('edit-suuplier/(:any)', 'Supplier::update/$1');
        $routes->get('delete-suuplier/(:any)', 'Supplier::delete/$1');
        
        $routes->get('{locale}/lang','Lang::index');
        $routes->post('{locale}/lang','Lang::index');


$routes->get('/customer', 'Customer::index',['filter' => 'role:admin'] );
$routes->addRedirect('/customer', 'customer/index' );

/*
 * --------------------------------------------------------------------
 * Additional Routing
 * --------------------------------------------------------------------
 *
 * There will often be times that you need additional routing and you
 * need it to be able to override any defaults in this file. Environment
 * based routes is one such time. require() additional route files here
 * to make that happen.
 *
 * You will have access to the $routes object within that file without
 * needing to reload it.
 */
if (is_file(APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php')) {
    require APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php';
}
